==> irreco-core_0.6.3-3.tar.gz: irreco-core-0.6.3/src/core/irreco_theme.c <==
/*
 * irreco - Ir Remote Control
 * Copyright (C) 2008 Joni Kokko (t5kojo01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_theme.h"

/**
 * @addtogroup IrrecoTheme
 * @ingroup Irreco
 *
 * Contains information of theme.
 *
 * @{ 
 */

/**
 * @file
 * Source file of @ref IrrecoTheme.
 */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void irreco_theme_read_button_keyfile_foreach(IrrecoDirForeachData * dir_data);
void irreco_theme_read_bg_keyfile_foreach(IrrecoDirForeachData * dir_data);
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Construction & Destruction
 * @{
 */

IrrecoTheme *irreco_theme_new(const char *name, const char *path)
{
	IrrecoTheme *self;
	IRRECO_ENTER
	
	self = g_slice_new0(IrrecoTheme);
	
	self->name = g_string_new(name);
	self->path = g_string_new(path);
	self->backgrounds =  irreco_string_table_new(
				(GDestroyNotify)irreco_theme_bg_free, NULL);
	self->buttons =  irreco_string_table_new(
				(GDestroyNotify)irreco_theme_button_free, NULL);
	
	IRRECO_DEBUG("Themepath: %s\n", path);

	/*Get buttons*/
	{
		IrrecoDirForeachData button_styles;
		GString * directory = g_string_new("");	
	
		g_string_printf(directory, "%s/buttons/", path);
		IRRECO_DEBUG("Directory = %s\n", directory->str);
		button_styles.directory = directory->str;
	
		button_styles.filesuffix = "bstyleconf";
		button_styles.user_data_1 = self;
	
		irreco_dir_foreach_subdirectories(&button_styles, 
				irreco_theme_read_button_keyfile_foreach);

		g_string_free(directory, TRUE);
		directory = NULL;

		irreco_string_table_sort_abc(self->buttons);
	}
	
	/*Get backgrounds*/
	{
		IrrecoDirForeachData bg_styles;
		GString * directory = g_string_new("");	
	
		g_string_printf(directory, "%s/bg/", path);
		IRRECO_DEBUG("Directory = %s\n", directory->str);
		bg_styles.directory = directory->str;

		bg_styles.filesuffix = "bgstyleconf";
		bg_styles.user_data_1 = self;

		irreco_dir_foreach_subdirectories(&bg_styles,
				irreco_theme_read_bg_keyfile_foreach);

		g_string_free(directory, TRUE);
		directory = NULL;

		irreco_string_table_sort_abc(self->backgrounds);
	}

	IRRECO_RETURN_PTR(self);
}

void irreco_theme_free(IrrecoTheme *self)
{
	IRRECO_ENTER
			
	g_assert(self != NULL);
	
	g_string_free(self->name, TRUE);
	self->name = NULL;
	
	g_string_free(self->path, TRUE);
	self->path = NULL;

	irreco_string_table_free(self->backgrounds);
	self->backgrounds = NULL;

	irreco_string_table_free(self->buttons);
	self->buttons = NULL;

	g_slice_free(IrrecoTheme, self);
	self = NULL;

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Public Functions                                                           */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Public Functions
 * @{
 */

void irreco_theme_print(IrrecoTheme *self)
{
	IRRECO_ENTER

	IRRECO_PRINTF("Themename: %s \n", self->name->str);
	IRRECO_PRINTF("Folder: %s \n", self->path->str);
	irreco_string_table_print(self->backgrounds);
	irreco_string_table_print(self->buttons);
	
	#if 0
	IRRECO_STRING_TABLE_FOREACH_DATA(self->backgrounds, void *, theme_bg)
		irreco_theme_button_print(theme_bg);
	IRRECO_STRING_TABLE_FOREACH_END

	IRRECO_STRING_TABLE_FOREACH_DATA(self->buttons, void *, theme_button)
		irreco_theme_button_print(theme_button);
	IRRECO_STRING_TABLE_FOREACH_END
	#endif

	IRRECO_RETURN
}

void irreco_theme_read_button_keyfile_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoTheme	*self		= (IrrecoTheme*) dir_data->user_data_1;
	IrrecoKeyFile	*keyfile;
	IrrecoButtonStyle *style;
	IrrecoThemeButton *theme_button;
	IRRECO_ENTER

	keyfile = irreco_keyfile_create(dir_data->directory, 
					dir_data->filepath, 
					"button-style");
	if (keyfile == NULL) goto end;
	
	style = irreco_button_style_create(NULL, NULL, NULL,
					   NULL, TRUE, FALSE);
	
	/* Required fields. */			
	if (!irreco_keyfile_get_str(keyfile, "name", &style->name) ||
	    !irreco_keyfile_get_path(keyfile, "up", &style->up)) {
		IRRECO_PRINTF("Could not read style from group \"%s\"\n", 
			      keyfile->group);
		irreco_button_style_destroy(style);
		IRRECO_RETURN
	}
	
	/* Optional fields. */
	irreco_keyfile_get_path(keyfile, "down", &style->down);
	irreco_keyfile_get_bool(keyfile, "allow-text", &style->allow_text);
	irreco_keyfile_get_str(keyfile, "text-format-up", &style->text_format_up);
	irreco_keyfile_get_str(keyfile, "text-format-down", 
			       &style->text_format_down);
	irreco_keyfile_get_int(keyfile, "text-padding", &style->text_padding);
	
	theme_button = irreco_theme_button_new();
	irreco_theme_button_set(theme_button,
				style->name, style->allow_text,
				style->up, style->down,
				style->text_format_up, style->text_format_down,
				style->text_padding);
	
	irreco_string_table_add(self->buttons,
				theme_button->name->str,
				theme_button);
	
	irreco_theme_button_print(theme_button);

	irreco_keyfile_destroy(keyfile);
	
	end:

	IRRECO_RETURN
}

void irreco_theme_read_bg_keyfile_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoTheme	*self = (IrrecoTheme*) dir_data->user_data_1;
	IrrecoKeyFile	*keyfile;
	IrrecoThemeBg	*bg;
	char *name;
	char *image;
	IRRECO_ENTER

	keyfile = irreco_keyfile_create(dir_data->directory,
					dir_data->filepath,
					"bg-style");
	if (keyfile == NULL) goto end;

	/* Required fields. */
	if (!irreco_keyfile_get_str(keyfile, "name", &name) ||
	    !irreco_keyfile_get_path(keyfile, "image", &image)) {
		IRRECO_PRINTF("Could not read style from group \"%s\"\n",
			      keyfile->group);
		goto end;
	}
	bg = irreco_theme_bg_new();

	irreco_theme_bg_set(bg, name, image);
	irreco_string_table_add(self->backgrounds, name, bg);

	irreco_theme_bg_print(bg);

	irreco_keyfile_destroy(keyfile);

	end:
	g_free(name);
	g_free(image);
	IRRECO_RETURN
}

IrrecoStringTable* irreco_theme_get_buttons(IrrecoTheme *self)
{
	IRRECO_ENTER		
	IRRECO_RETURN_PTR(self->buttons);
}

IrrecoStringTable* irreco_theme_get_backgrounds(IrrecoTheme *self)
{
	IRRECO_ENTER		
	IRRECO_RETURN_PTR(self->backgrounds);
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/** @} */

==> irreco-core_0.6.3-4.tar.gz: irreco-core-0.6.3/src/core/irreco_theme.c <==
/*
 * irreco - Ir Remote Control
 * Copyright (C) 2008 Joni Kokko (t5kojo01@students.oamk.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_theme.h"

/**
 * @addtogroup IrrecoTheme
 * @ingroup Irreco
 *
 * Contains information of theme.
 *
 * @{ 
 */

/**
 * @file
 * Source file of @ref IrrecoTheme.
 */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void irreco_theme_read_button_keyfile_foreach(IrrecoDirForeachData * dir_data);
void irreco_theme_read_bg_keyfile_foreach(IrrecoDirForeachData * dir_data);
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Construction & Destruction
 * @{
 */

IrrecoTheme *irreco_theme_new(const char *name, const char *path)
{
	IrrecoTheme *self;
	IRRECO_ENTER
	
	self = g_slice_new0(IrrecoTheme);
	
	self->name = g_string_new(name);
	self->path = g_string_new(path);
	self->backgrounds =  irreco_string_table_new(
				(GDestroyNotify)irreco_theme_bg_free, NULL);
	self->buttons =  irreco_string_table_new(
				(GDestroyNotify)irreco_theme_button_free, NULL);
	
	IRRECO_DEBUG("Themepath: %s\n", path);

	/*Get buttons*/
	{
		IrrecoDirForeachData button_styles;
		GString * directory = g_string_new("");	
	
		g_string_printf(directory, "%s/buttons/", path);
		IRRECO_DEBUG("Directory = %s\n", directory->str);
		button_styles.directory = directory->str;
	
		button_styles.filesuffix = "button.conf";
		button_styles.user_data_1 = self;
	
		irreco_dir_foreach_subdirectories(&button_styles, 
				irreco_theme_read_button_keyfile_foreach);

		g_string_free(directory, TRUE);
		directory = NULL;

		irreco_string_table_sort_abc(self->buttons);
	}
	
	/*Get backgrounds*/
	{
		IrrecoDirForeachData bg_styles;
		GString * directory = g_string_new("");	
	
		g_string_printf(directory, "%s/bg/", path);
		IRRECO_DEBUG("Directory = %s\n", directory->str);
		bg_styles.directory = directory->str;

		bg_styles.filesuffix = "bg.conf";
		bg_styles.user_data_1 = self;

		irreco_dir_foreach_subdirectories(&bg_styles,
				irreco_theme_read_bg_keyfile_foreach);

		g_string_free(directory, TRUE);
		directory = NULL;

		irreco_string_table_sort_abc(self->backgrounds);
	}

	IRRECO_RETURN_PTR(self);
}

void irreco_theme_free(IrrecoTheme *self)
{
	IRRECO_ENTER
			
	g_assert(self != NULL);
	
	g_string_free(self->name, TRUE);
	self->name = NULL;
	
	g_string_free(self->path, TRUE);
	self->path = NULL;

	irreco_string_table_free(self->backgrounds);
	self->backgrounds = NULL;

	irreco_string_table_free(self->buttons);
	self->buttons = NULL;

	g_slice_free(IrrecoTheme, self);
	self = NULL;

	IRRECO_RETURN
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Public Functions                                                           */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * @name Public Functions
 * @{
 */

void irreco_theme_print(IrrecoTheme *self)
{
	IRRECO_ENTER

	IRRECO_PRINTF("Themename: %s \n", self->name->str);
	IRRECO_PRINTF("Folder: %s \n", self->path->str);
	irreco_string_table_print(self->backgrounds);
	irreco_string_table_print(self->buttons);
	
	#if 0
	IRRECO_STRING_TABLE_FOREACH_DATA(self->backgrounds, void *, theme_bg)
		irreco_theme_button_print(theme_bg);
	IRRECO_STRING_TABLE_FOREACH_END

	IRRECO_STRING_TABLE_FOREACH_DATA(self->buttons, void *, theme_button)
		irreco_theme_button_print(theme_button);
	IRRECO_STRING_TABLE_FOREACH_END
	#endif

	IRRECO_RETURN
}

void irreco_theme_read_button_keyfile_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoTheme	*self		= (IrrecoTheme*) dir_data->user_data_1;
	IrrecoKeyFile	*keyfile;
	IrrecoButtonStyle *style;
	IrrecoThemeButton *theme_button;
	GString	*style_name = g_string_new(self->name->str);
	IRRECO_ENTER

	keyfile = irreco_keyfile_create(dir_data->directory, 
					dir_data->filepath, 
					"theme-button");
	if (keyfile == NULL) goto end;
	
	style = irreco_button_style_create(NULL, NULL, NULL,
					   NULL, TRUE, FALSE);
	
	/* Required fields. */			
	if (!irreco_keyfile_get_str(keyfile, "name", &style->name) ||
	    !irreco_keyfile_get_path(keyfile, "up", &style->up)) {
		IRRECO_PRINTF("Could not read style from group \"%s\"\n", 
			      keyfile->group);
		irreco_button_style_destroy(style);
		IRRECO_RETURN
	}
	
	/* Optional fields. */
	irreco_keyfile_get_path(keyfile, "down", &style->down);
	irreco_keyfile_get_bool(keyfile, "allow-text", &style->allow_text);
	irreco_keyfile_get_str(keyfile, "text-format-up", &style->text_format_up);
	irreco_keyfile_get_str(keyfile, "text-format-down", 
			       &style->text_format_down);
	irreco_keyfile_get_int(keyfile, "text-padding", &style->text_padding);
	
	g_string_append_printf(style_name,"/%s", style->name);
	theme_button = irreco_theme_button_new();
	irreco_theme_button_set(theme_button, style_name->str,
				style->name, style->allow_text,
				style->up, style->down,
				style->text_format_up, style->text_format_down,
				style->text_padding);
	
	irreco_string_table_add(self->buttons,
				theme_button->style_name->str,
				theme_button);
	
	irreco_theme_button_print(theme_button);

	irreco_keyfile_destroy(keyfile);
	
	end:
	g_string_free(style_name, TRUE);
	IRRECO_RETURN
}

void irreco_theme_read_bg_keyfile_foreach(IrrecoDirForeachData * dir_data)
{
	IrrecoTheme	*self = (IrrecoTheme*) dir_data->user_data_1;
	IrrecoKeyFile	*keyfile;
	IrrecoThemeBg	*bg;
	char *name;
	char *image;
	IRRECO_ENTER

	keyfile = irreco_keyfile_create(dir_data->directory,
					dir_data->filepath,
					"theme-bg");
	if (keyfile == NULL) goto end;

	/* Required fields. */
	if (!irreco_keyfile_get_str(keyfile, "name", &name) ||
	    !irreco_keyfile_get_path(keyfile, "image", &image)) {
		IRRECO_PRINTF("Could not read style from group \"%s\"\n",
			      keyfile->group);
		goto end;
	}
	bg = irreco_theme_bg_new();

	irreco_theme_bg_set(bg, name, image);
	irreco_string_table_add(self->backgrounds, name, bg);

	irreco_theme_bg_print(bg);

	irreco_keyfile_destroy(keyfile);

	end:
	g_free(name);
	g_free(image);
	IRRECO_RETURN
}

IrrecoStringTable* irreco_theme_get_buttons(IrrecoTheme *self)
{
	IRRECO_ENTER		
	IRRECO_RETURN_PTR(self->buttons);
}

IrrecoStringTable* irreco_theme_get_backgrounds(IrrecoTheme *self)
{
	IRRECO_ENTER		
	IRRECO_RETURN_PTR(self->backgrounds);
}

/** @} */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/** @} */
